;;======================================================================;;
;;			SERVO TESTER				;;
;;======================================================================;;
;;									;;
;; Program:         SERVOTST -- Tester for model servo	;;
;; Code:            Jindra Fuk						;;
;; Platform:        Microchip PIC12F629, 4 Mhz				;;
;; Date:            29.05.2015						;;
;; First release:   29.05.2015						;;
;; LastDate:        29.05.2015						;;
;; Updated:         Jindra Fuk                                         ;;
;; UpdateDate:      29.05.2015                                           ;;
;;									;;
;;======================================================================;;

; Minimal external components, uses internal oscilator at 4 MHz

; This program is distributed as is but WITHOUT ANY WARRANTY
; I hope you enjoy!!
;
; Revisions:
; 29.05.2015	Start of writting code

; ----- Definitions

#define		__VERNUM	D'1'
#define		__VERDAY	0x29
#define		__VERMONTH	0x05
#define		__VERYEAR	0x15


	LIST	   p=12F629	; target processor

	errorlevel -302		; Register in operand not in bank 0.  Ensure that bank bits are correct.
	;errorlevel -305	; Using default destination of 1 (file).

	#include p12F629.inc


	__CONFIG  _BODEN_ON & _CP_OFF & _WDT_OFF & _MCLRE_OFF & _PWRTE_OFF & _INTRC_OSC_NOCLKOUT 

				; Make sure that internal osc. is calibrated
				; Value has to be read before reprogramming the device.


;            +------U------+
;       +5V 1|Vdd       Vss|8 GND
; servo - o 2|GP5       GP0|7 io - LED1 \
; btn A - i 3|GP4       GP1|6 io - LED2 -- Charlieplexing
; btn B - i 4|Mclr(GP3) GP2|5 io - LED3 /
;            +-------------+

; --- Constant values

FXTAL		equ	D'4000000'		; internal oscilator

GP_TRIS         equ     b'00011000'		; GP0-2 io,GP3-4: inputs, GP5 output
GP_INI          equ     0x00			; all zero
OPTION_INI	equ	0x88					; Option register: no pull-up, falling GP2, no prescaler, wdt 1:1
WPU_INI		equ	0x33					; Weak pull-up enable. default, no pull-ups

INTC_INI	equ	b'11000000'				; GIE, INTE enable, PEIE enable
PIE1_INI	equ	b'00000001'				; interrupt TMR1


#define		SERVO	GPIO,5			; Servo output
#define		BTN_A	GPIO,4			; Button A
#define		BTN_B	GPIO,3			; Button B

; --- EEPROM Section

#define		EE_INI		0x00

E_CV0		equ	EE_INI+0x00		; CV1 - first servo position (*10 in ms) - default 90-> 900ms
E_CV1		equ	EE_INI+0x01		; CV2 - second servo position (*10 in ms) - default 110-> 1100ms
E_CV2		equ	EE_INI+0x02		; CV3 - second servo position (*10 in ms) - default 150-> 1500ms (default)
E_CV3		equ	EE_INI+0x03		; CV4 - second servo position (*10 in ms) - default 190-> 1900ms
E_CV4		equ	EE_INI+0x04		; CV5 - second servo position (*10 in ms) - default 210-> 2100ms
E_CV5		equ	EE_INI+0x05		; CV6 - Spacing
E_CV6		equ	EE_INI+0x06		; CV7 - Speed

; ----- Variables

; --- Internal RAM Section

        cblock  0x20

INT_W			; interrupt context registers
INT_STAT
EEDATA0

FLAGS
TEMP
COUNT
STATE	; servo state

CV0		; CV1 - first servo position (*10 in ms) - default 90-> 900ms
CV1		; CV2 - second servo position (*10 in ms) - default 110-> 1100ms
CV2		; CV3 - second servo position (*10 in ms) - default 150-> 1500ms (default)
CV3		; CV4 - second servo position (*10 in ms) - default 190-> 1900ms
CV4		; CV5 - second servo position (*10 in ms) - default 210-> 2100ms
CV5		; CV6 - Spacing
CV6		; CV7 - Speed

CV6CNT	; - Speed counter

POS		; current position of move for servo
DEST	; destination of move for servo
PULSE		; pulse duration
SPACE_H		; spacing duration
SPACE_L

DEBOUNCE	; keyboard debounce and auto repeat
		endc

; --- Flags

#define		DO_PULSE	FLAGS,0		; do pulse, TMR1 end

; --------------- Program Section --------------------------------------


		org	0x000

PowerUp:
		clrf	STATUS			; Bank 0 default
		clrf	INTCON			; Disable all interrupts
		clrf	PCLATH			; tables on page 0
		goto	INIT

; ----------------------------------------------------------------------

		org	0x004

Interrupt:
		movwf	INT_W			; save context registers
		swapf	STATUS,w
		movwf	INT_STAT
		clrf	STATUS			; interrupt uses bank 0

		bcf	SERVO			; yes, clear ports
		bcf	PIR1,TMR1IF
		bsf	DO_PULSE

		swapf	INT_STAT,w		; restore context registers	;49	;34
		movwf	STATUS							;50	;35
		swapf	INT_W,f							;51	;36
		swapf	INT_W,w							;52	;37
		retfie								;53,54	;38,39

; ----------------------------------------------------------------------

;Charlieplexing







ChplxPos:
		andlw	0x07
		addwf	PCL,f
						;LED- A K |tris|out
		retlw	0x41	; 0 - 0 1 | 100|001
		retlw	0x42	; 1 - 1 0 | 100|010
		retlw	0x21	; 2 - 0 2 | 010|001
		retlw	0x24	; 3 - 2 0 | 010|100
		retlw	0x12	; 4 - 1 2 | 001|010
		retlw	0x14	; 5 - 2 1 | 001|100
		retlw	0x70
		retlw	0x70

; ----------------------------------------------------------------------

INIT:
		clrf	GPIO
		movlw	0x07
		movwf	CMCON			; set GP2:0 to digital I/O

		bsf	STATUS,RP0		; bank 1
		movlw	GP_TRIS
		movwf	TRISIO
		call	0x3FF			; get OSCCAL value
		movwf	OSCCAL
		movlw	WPU_INI			; pull-ups
		movwf	WPU
		clrf	IOC			; interrupt on change
		clrf	VRCON			; voltage reference off
		movlw	OPTION_INI		; Option register: no pull-up, falling GP2, no prescaler, wdt 1:1
		movwf	OPTION_REG
		movlw	PIE1_INI
		movwf	PIE1
		bcf	STATUS,RP0		; bank 0
		clrf	PIR1
		movlw	0x01			; Timer 1 on, 1:1 , internal 
		movwf	T1CON

		movlw	0x20			; clear RAM
		movwf	FSR
ClearRAM:
		clrf	INDF
		incf	FSR,f
		movlw	0x60
		xorwf	FSR,w
		btfss	STATUS,Z
		goto	ClearRAM

		movlw	0x90
		movwf	TMR1L
		movlw	0xFF
		movwf	TMR1H
		bcf	PIR1,TMR1IF

		movlw	INTC_INI                ; Int enable, enable peripheral int, disable tmr0 int, enable GP2 int, disable GPIO change int, 
		movwf	INTCON			; enable GP2 external interrupt

		call	LoadCV			; load CV values

		movf	CV2,w
		movwf	PULSE
		movwf	DEST
		movlw	1
		movwf	CV6CNT
		movlw	0x02
		movwf	POS

		call	ChplxPos
		movwf	TEMP
		clrf	GPIO			; clear GPIO to blank leds
		swapf	TEMP,w
		andlw	0x0F			; upper nible is TRIS
		iorlw	GP_TRIS			; add default mask
		bsf	STATUS,RP0			; bank 1
		movwf	TRISIO
		bcf	STATUS,RP0			; bank 0
		movf	TEMP,w
		andlw	0x0F			; low nibble is output
		movwf	GPIO			; to GPIO
		

; ----------------------------------------------------------------------

MainLoop:
		btfsc	DO_PULSE		; end of servo pulse?
		call	DoServo			; yes, next pulse

		goto	MainLoop

; ----------------------------------------------------------------------

; ----------------------------------------------------------------------

DoServo:
		bcf	DO_PULSE

		btfsc	STATE,0
		goto	Spacing

PulseServo1:
		;bcf	T1CON,TMR1ON
		movf	CV5,w			; init spacing value
		sublw	0x00
		movwf	SPACE_H
		clrf	SPACE_L

		movf	PULSE,w		; 200: 2ms, 100: 1ms
		call	PulseServo

		bsf	SERVO

		bsf	T1CON,TMR1ON		; run timer 1
		incf	STATE,f

		decfsz	CV6CNT,f		; next speed step
		return					; yes, then do nothing

		movf	CV6,w               ; finished = prepare next step
		movwf	CV6CNT
                 
		movf	PULSE,w
		subwf   DEST,w			; dest - pulse --> w ( if dest<pulse then C, if dest=pulse then Z)

		btfss	STATUS,C		; going down ???
		goto	EndServoDec		; yes - then decrease
                 
		btfss	STATUS,Z		; going up ???
		goto	EndServoInc		; yes - then increase

		movf	POS,w
		xorlw	0x05			; is position 5 = pendling??
		btfss	STATUS,Z
		return					; no - do nothing
		
		movf	PULSE,w			; is on position 0??
		xorwf	CV0,w
		btfsc	STATUS,Z
		goto	Pulse4
		movf	CV0,w
		movwf	DEST
		return
Pulse4:
		movf	CV4,w
		movwf	DEST
		return

EndServoDec:
		decf	PULSE,f
		btfss	CV6,7				; if bit 7 is set, then handle it
		return
		movlw	1						; set speed factor to 1
		movwf	CV6CNT
		movf	CV6,w				
		andlw	0x7F					; remove marking one from CV515
		subwf	PULSE,f				; pulse1 = pulse1 - cv515
        return

EndServoInc:
		incf	PULSE,f
		btfss	CV6,7				; if bit 7 is set, then handle it
        return
		movlw	1						; set speed factor to 1
		movwf	CV6CNT
		movf	CV6,w				
		andlw	0x7F					; remove marking one from CV515
		addwf	PULSE,f				; pulse1 = pulse1 - cv515
        return
                 

Spacing:
		bcf	T1CON,TMR1ON
		movf	SPACE_H,w
		movwf	TMR1H
		movf	SPACE_L,w
		movwf	TMR1L
		bsf	T1CON,TMR1ON		; run timer 1
		incf	STATE,f
		
		comf	GPIO,w
		andlw	0x18			; only buttons
		movwf	TEMP
		btfss	STATUS,Z
		goto	BtnPress
		movlw	3
		movwf	DEBOUNCE
		return

PulseServo:
		bcf	T1CON,TMR1ON		; stop timer 1
		clrf	TMR1H			; do x10
		movwf	TEMP
		movwf	TMR1L
		bcf     STATUS,C
		rlf	TMR1L,f                 ; x2
		rlf	TMR1H,f                 
		rlf	TMR1L,f			; x4
		rlf	TMR1H,f
		movf	TEMP,w			; x4 + 1 = x5
		addwf	TMR1L,f
		btfsc	STATUS,C
		incf	TMR1H,f
                 bcf     STATUS,C
		rlf	TMR1L,f			; x5 * 2 = x10
		rlf	TMR1H,f

		movf	TMR1L,w			; correct spacing
		addwf	SPACE_L,f
		btfsc	STATUS,C
		incf	SPACE_H,f
		movf	TMR1H,w
		addwf	SPACE_H,f

		comf	TMR1L,f			; negative for incrementing
		comf	TMR1H,f
		movlw	0x01
		addwf	TMR1L,f
		btfsc	STATUS,C
		incf	TMR1H,f
		return

BtnPress:
		decfsz	DEBOUNCE,f
		return

		movlw	0x30
		movwf	DEBOUNCE

		movf	TEMP,w
		xorlw	0x18			; both buttons ???
		btfss	STATUS,Z
		goto	OneBtnPress
		movf	CV2,w
		movwf	PULSE
		movwf	DEST
		movlw	0x02
		movwf	POS
DispPos
		movf	POS,w
		call	ChplxPos
		movwf	TEMP
		clrf	GPIO			; clear GPIO to blank leds
		swapf	TEMP,w
		andlw	0x0F			; upper nible is TRIS
		iorlw	GP_TRIS			; add default mask
		bsf	STATUS,RP0			; bank 1
		movwf	TRISIO
		bcf	STATUS,RP0			; bank 0
		movf	TEMP,w
		andlw	0x0F			; low nibble is output
		movwf	GPIO			; to GPIO
		return
		
OneBtnPress:
		btfsc	TEMP,3
		incf	POS,f
		btfsc	TEMP,4
		decf	POS,f

		movf	POS,w
		xorlw	0x06
		btfss	STATUS,Z
		goto	FFBtnPress
		clrf	POS
		movf	CV0,w
		movwf	PULSE
		movwf	DEST
		goto	DispPos

FFBtnPress
		incf	POS,w
		btfss	STATUS,Z
		goto	SetPos
		movlw	0x05
		movwf	POS
SetPos5:
		movf	CV0,w
		movwf	DEST
		movlw	1
		movwf	CV6CNT
		goto	DispPos

SetPos
		movf	POS,w
		xorlw	0x05
		btfsc	STATUS,Z
		goto	SetPos5
		movlw	CV0
		addwf	POS,w
		movwf	FSR
		movf	INDF,w
		movwf	PULSE
		movwf	DEST
		goto	DispPos
		
;----------------------------------------------------------------------


LoadCV:

		movlw	E_CV0			; CV1 - first servo position (*10 in ms) - default 90-> 900ms
		call	EE_Read
		movwf	CV0
		movlw	E_CV1			; CV2 - second servo position (*10 in ms) - default 110-> 1100ms
		call	EE_Read
		movwf	CV1
		movlw	E_CV2			; CV3 - third servo position (*10 in ms) - default 150-> 1500ms (default)
		call	EE_Read
		movwf	CV2
		movlw	E_CV3			; CV4 - fourth servo position (*10 in ms) - default 190-> 1900ms
		call	EE_Read
		movwf	CV3
		movlw	E_CV4			; CV5 - fifth servo position (*10 in ms) - default 210-> 2100ms
		call	EE_Read
		movwf	CV4
		movlw	E_CV5			; CV6 - Spacing
		call	EE_Read
		movwf	CV5
		movlw	E_CV6			; CV7 - Speed
		call	EE_Read
		movwf	CV6

		return

;---------------------------------------------------------------------------

EE_Read:
		bsf	STATUS,RP0		; w=ADR
		movwf	EEADR
		bsf	EECON1,RD
		movf	EEDATA,w
		bcf	STATUS,RP0
		return

SetParm:
		call	EE_Read			; w=ADR, EEDATA0=data. Write only changes
		xorwf	EEDATA0,w
		btfsc	STATUS,Z
		return
EE_Write:		
		movf	EEDATA0,w
		bsf	STATUS,RP0
		movwf	EEDATA
		bsf	EECON1,WREN
		bcf	INTCON,GIE
		movlw	0x55
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf	EECON1,WR
		bsf	INTCON,GIE
		bcf	EECON1,WREN
EEWrite0:
		btfsc	EECON1,WR
		goto	EEWrite0
		bcf	STATUS,RP0
		return



; ----- EEPROM default values


		org	0x2100

		dw	.90			; CV1 - first servo position (*10 in ms) - default 90-> 900ms
		dw	.110		; CV2 - second servo position (*10 in ms) - default 110-> 1100ms
		dw	.150		; CV3 - third servo position (*10 in ms) - default 150-> 1500ms (default)
		dw	.190		; CV4 - fourth servo position (*10 in ms) - default 190-> 1900ms
		dw	.210		; CV5 - fifth servo position (*10 in ms) - default 210-> 2100ms
		dw	0x4E		; CV6 - Spacing
		dw	0x01		; CV7 - Speed


		org	0x2110

		dt	" servo  "
		dt	"  test  "
		dt	"J.Fucik "
		dt	(__VERDAY   >> 4)  +0x30
		dt	(__VERDAY   & 0x0F)+0x30,"/"
		dt	(__VERMONTH >> 4)  +0x30
		dt	(__VERMONTH & 0x0F)+0x30,"/"
		dt	(__VERYEAR  >> 4)  +0x30
		dt	(__VERYEAR  & 0x0F)+0x30

	end